@@ -1,8 +1,12 @@
+from __future__ import unicode_literals
+
 import sys
 
 import django
 from django.conf import settings
 from django.utils.translation import ugettext_lazy as _
+from django.utils.datastructures import SortedDict
+from django.utils.importlib import import_module
 
 from debug_toolbar.panels import DebugPanel
 
@@ -14,26 +18,24 @@ class VersionDebugPanel(DebugPanel):
     name = 'Version'
     template = 'debug_toolbar/panels/versions.html'
     has_content = True
-    
+
     def nav_title(self):
         return _('Versions')
-    
+
     def nav_subtitle(self):
         return 'Django %s' % django.get_version()
-    
+
     def url(self):
         return ''
-    
+
     def title(self):
         return _('Versions')
-    
+
     def process_response(self, request, response):
-        versions = {}
-        versions['Python'] = '%d.%d.%d' % sys.version_info[:3]
+        versions = [('Python', '%d.%d.%d' % sys.version_info[:3])]
         for app in list(settings.INSTALLED_APPS) + ['django']:
             name = app.split('.')[-1].replace('_', ' ').capitalize()
-            __import__(app)
-            app = sys.modules[app]
+            app = import_module(app)
             if hasattr(app, 'get_version'):
                 get_version = app.get_version
                 if callable(get_version):
@@ -48,9 +50,10 @@ class VersionDebugPanel(DebugPanel):
                 continue
             if isinstance(version, (list, tuple)):
                 version = '.'.join(str(o) for o in version)
-            versions[name] = version
-        
+            versions.append((name, version))
+            versions = sorted(versions, key=lambda version: version[0])
+
         self.record_stats({
-            'versions': versions,
+            'versions': SortedDict(versions),
             'paths': sys.path,
         })
