@@ -1,27 +1,52 @@
+from __future__ import unicode_literals
+
 import inspect
 import os.path
 import django
-import SocketServer
 import sys
 
-from django.conf import settings
+from django.core.exceptions import ImproperlyConfigured
+from django.utils.encoding import force_text
+from django.utils.html import escape
+from django.utils.importlib import import_module
+from django.utils.safestring import mark_safe
+from django.utils import six
 from django.views.debug import linebreak_iter
 
+from .settings import CONFIG
+
 # Figure out some paths
 django_path = os.path.realpath(os.path.dirname(django.__file__))
-socketserver_path = os.path.realpath(os.path.dirname(SocketServer.__file__))
 
-def ms_from_timedelta(td):
-    """
-    Given a timedelta object, returns a float representing milliseconds
-    """
-    return (td.seconds * 1000) + (td.microseconds / 1000.0)
+
+def get_module_path(module_name):
+    try:
+        module = import_module(module_name)
+    except ImportError as e:
+        raise ImproperlyConfigured(
+            'Error importing HIDDEN_STACKTRACE_MODULES: %s' % (e,))
+    else:
+        source_path = inspect.getsourcefile(module)
+        if source_path.endswith('__init__.py'):
+            source_path = os.path.dirname(source_path)
+        return os.path.realpath(source_path)
+
+
+hidden_paths = [
+    get_module_path(module_name)
+    for module_name in CONFIG['HIDDEN_STACKTRACE_MODULES']
+]
+
+
+def omit_path(path):
+    return any(path.startswith(hidden_path) for hidden_path in hidden_paths)
+
 
 def tidy_stacktrace(stack):
     """
     Clean up stacktrace and remove all entries that:
     1. Are part of Django (except contrib apps)
-    2. Are part of SocketServer (used by Django's dev server)
+    2. Are part of socketserver (used by Django's dev server)
     3. Are the last entry (which is part of our stacktracing code)
 
     ``stack`` should be a list of frame tuples from ``inspect.stack()``
@@ -33,18 +58,33 @@ def tidy_stacktrace(stack):
         # inspection.
         if '__traceback_hide__' in frame.f_locals:
             continue
-        if getattr(settings, 'DEBUG_TOOLBAR_CONFIG', {}).get('HIDE_DJANGO_SQL', True) \
-            and django_path in s_path and not 'django/contrib' in s_path:
+        if CONFIG['HIDE_DJANGO_SQL'] and django_path in s_path and not 'django/contrib' in s_path:
             continue
-        if socketserver_path in s_path:
+        if omit_path(s_path):
             continue
-        if not text:
-            text = ''
-        else:
-            text = (''.join(text)).strip()
+        text = (''.join(force_text(t) for t in text)).strip() if text else ''
         trace.append((path, line_no, func_name, text))
     return trace
 
+
+def render_stacktrace(trace):
+    stacktrace = []
+    for frame in trace:
+        params = map(escape, frame[0].rsplit(os.path.sep, 1) + list(frame[1:]))
+        params_dict = dict((six.text_type(idx), v) for idx, v in enumerate(params))
+        try:
+            stacktrace.append('<span class="path">%(0)s/</span>'
+                              '<span class="file">%(1)s</span>'
+                              ' in <span class="func">%(3)s</span>'
+                              '(<span class="lineno">%(2)s</span>)\n'
+                              '  <span class="code">%(4)s</span>'
+                              % params_dict)
+        except KeyError:
+            # This frame doesn't have the expected format, so skip it and move on to the next one
+            continue
+    return mark_safe('\n'.join(stacktrace))
+
+
 def get_template_info(source, context_lines=3):
     line = 0
     upto = 0
@@ -79,6 +119,7 @@ def get_template_info(source, context_lines=3):
         'context': context,
     }
 
+
 def get_name_from_obj(obj):
     if hasattr(obj, '__name__'):
         name = obj.__name__
@@ -93,6 +134,7 @@ def get_name_from_obj(obj):
 
     return name
 
+
 def getframeinfo(frame, context=1):
     """
     Get information about a frame or traceback object.
@@ -116,15 +158,15 @@ def getframeinfo(frame, context=1):
 
     filename = inspect.getsourcefile(frame) or inspect.getfile(frame)
     if context > 0:
-        start = lineno - 1 - context//2
+        start = lineno - 1 - context // 2
         try:
             lines, lnum = inspect.findsource(frame)
-        except (IOError, IndexError):
+        except Exception:   # findsource raises platform-dependant exceptions
             lines = index = None
         else:
             start = max(start, 1)
             start = max(0, min(start, len(lines) - context))
-            lines = lines[start:start+context]
+            lines = lines[start:(start + context)]
             index = lineno - 1 - start
     else:
         lines = index = None
@@ -134,6 +176,7 @@ def getframeinfo(frame, context=1):
     else:
         return (filename, lineno, frame.f_code.co_name, lines, index)
 
+
 def get_stack(context=1):
     """
     Get a list of records for a frame and all higher (calling) frames.
@@ -148,4 +191,4 @@ def get_stack(context=1):
     while frame:
         framelist.append((frame,) + getframeinfo(frame, context))
         frame = frame.f_back
-    return framelist
\ No newline at end of file
+    return framelist
